﻿using System;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Controls;
using System.Windows.Media;
using ThinkGeo.MapSuite.SilverlightCore;
using ThinkGeo.MapSuite.SilverlightEdition;

namespace SilverlightApplication {

    [ScriptableTypeAttribute]
    public partial class MainPage : UserControl {
        public MainPage() {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(MainPage_Loaded);
        }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            Map1.MapUnit = GeographyUnit.DecimalDegree;
            NewServerLayerOverlay serverLayerOverlay = new NewServerLayerOverlay("NativeServer", "SilverlightMapConnector1");

            ProgressBar bar = new ProgressBar();
            bar.IsIndeterminate = true;
            bar.BorderBrush = new SolidColorBrush(Colors.Black);
            bar.BorderThickness = new Thickness(2d);
            bar.Foreground = new SolidColorBrush(Colors.Orange);
            bar.Minimum = 0;
            bar.Maximum = 100;
            bar.Width = 400;
            bar.Height = 30;

            serverLayerOverlay.LoadingCanvas.Width = 400;
            serverLayerOverlay.LoadingCanvas.Children.Add(bar);

            serverLayerOverlay.IsStretchEffectDisabled = true;
            Map1.Overlays.Add(serverLayerOverlay);
            Map1.CurrentExtent = new RectangleShape(-135.7, 83.6, 113.5, -53);
        }
    }

    public class NewServerLayerOverlay : ServerLayerOverlay
    {
        private Canvas loadingCanvas;
        private int tileCount;

        public NewServerLayerOverlay()
            : base()
        {
        }

        public NewServerLayerOverlay(string id, string connectorId)
            : base(id, connectorId)
        {
            loadingCanvas = new Canvas();
        }


        public Canvas LoadingCanvas
        {
            get { return loadingCanvas; }
            set { loadingCanvas = value; }
        }

        protected override void InitializeCore()
        {
            base.InitializeCore();

            Canvas overlayCanvas = (Canvas)OverlayElement;
            loadingCanvas.SetValue(Canvas.ZIndexProperty, 9999);
            if (!double.IsNaN(overlayCanvas.Height))
            {
                loadingCanvas.SetValue(Canvas.TopProperty, overlayCanvas.Height / 2);
            }
            if (!double.IsNaN(overlayCanvas.Width))
            {
                double left = overlayCanvas.Width / 2;
                if (!double.IsNaN(loadingCanvas.ActualWidth))
                {
                    left -= loadingCanvas.ActualWidth / 2;
                }
                loadingCanvas.SetValue(Canvas.LeftProperty, left);
            }
            overlayCanvas.Children.Add(loadingCanvas);
        }

        protected override void DrawCore(RectangleShape worldExtent)
        {
            base.DrawCore(worldExtent);

            loadingCanvas.Visibility = Visibility.Visible;
            tileCount = 0;

            Canvas overlayCanvas = (Canvas)OverlayElement;
            foreach (UIElement element in overlayCanvas.Children)
            {
                Canvas canvas = element as Canvas;
                if (canvas != null)
                {
                    foreach (UIElement child in canvas.Children)
                    {
                        Image image = child as Image;
                        if (image != null)
                        {
                            tileCount++;
                            image.ImageOpened += new EventHandler<RoutedEventArgs>(image_ImageOpened);
                        }
                    }
                }
            }
        }

        void image_ImageOpened(object sender, RoutedEventArgs e)
        {
            tileCount--;
            if (tileCount <= 0)
            {
                loadingCanvas.Visibility = Visibility.Collapsed;
            }
        }
    }
}
